@@ -9,33 +9,39 @@ import os.path
 import subprocess
 import sys
 import time
+import thread
 
 
-def run_subprocess(params):
+def run_subprocess(quite, params):
     "Run the given pramams in a subprocess."
     try:
         subprocess.Popen(params)
     except OSError, e:
         # On error it would be nice to have a hint why it failed:
-        print "OSError: %s" % (e.args,)
-        print "Popen params were: ",
-        print params
+        print "OSError: [Errno %s] %s" % e.args
+        if  quite:
+            print "Parameters were: %s" % " ".join(params)
         # Exit the observer on error.
         sys.exit(-1)
 
 
-
-def directoryCallbackFactory(*params):
+def directoryCallbackFactory(quite, *params):
     "Create callback function for directory events."
     def callback(subpath, mask):
         # Subpath and mask of changes do not matter here. There is
         # currently no way (and no desire) to handle them over to the
         # subprocess.
-        run_subprocess(params)
+        if not quite:
+            print "Calling: %s" % " ".join(params)
+        try:
+            run_subprocess(quite, params)
+        except SystemExit:
+            # Signal the process to exit as we are in a thread here.
+            thread.interrupt_main()
     return callback
 
 
-def fileCallbackFactory(extensions, *params):
+def fileCallbackFactory(extensions, quite, *params):
     "Create callback function for file events."
     def callback(event):
         # event.name contains the absolute path to the changed file
@@ -43,25 +49,37 @@ def fileCallbackFactory(extensions, *params):
         if ext in extensions:
             # Only run process when extension of changed file is in
             # the list of observed file types
-            run_subprocess(params)
+            if not quite:
+                print "Calling: %s" % " ".join(params)
+            try:
+                run_subprocess(quite, params)
+            except SystemExit:
+                # We are in a thread here, so signal the main thread to exit.
+                thread.interrupt_main()
     return callback
 
 
 class Observer(object):
     "Observer for path."
-    def __init__(self, path, params, extensions):
+
+    extensions = [] # only call utility when a file with this ext changed
+    quite = False # when True, do not print any non-error output
+
+    def __init__(self, path, params, **options):
         self.path = path
         self.params = params
-        self.extensions = extensions
         self._is_running = False
+        for key, value in options.items():
+            setattr(self, key, value)
 
     def start(self):
         if self.extensions:
             # observe explicit file extensions
-            callback = fileCallbackFactory(self.extensions, *self.params)
+            callback = fileCallbackFactory(
+                self.extensions, self.quite, *self.params)
         else:
             # observe everything in the directory
-            callback = directoryCallbackFactory(*self.params)
+            callback = directoryCallbackFactory(self.quite, *self.params)
         self.observer = fsevents.Observer()
         self.observer.start()
         file_events = bool(self.extensions)
@@ -88,9 +106,15 @@ def mangle_call_args(args, argv):
         "%prog invokes <utility> with its <utility arguments> when <path> or "
         "something in it changes.")
     parser.add_option(
-        "-e", action="append", dest="extension", default=[],
+        "-e", action="append", metavar="EXTENSION", dest="extensions",
+        default=[],
         help="only call utility on changes of a file with this extension "\
              "(option might be used multiple times)")
+    parser.add_option(
+        "-q", action="store_true", dest="quite", default=False,
+        help=(
+            "Do not display any output of callonchange. "
+            "(Still displays the output of the utility.)"))
     parser.disable_interspersed_args()
 
     (options, parsed_args) = parser.parse_args(call_args)
@@ -105,9 +129,10 @@ def mangle_call_args(args, argv):
     # specified extension, so they get added here, as they are needed
     # by the observer.
     extensions = []
-    for ext in options.extension:
+    for ext in options.extensions:
         if not ext.startswith('.'):
             ext = '.' + ext
         extensions.append(ext)
+    options.extensions = extensions
 
-    return parsed_args[0], parsed_args[1:], extensions
+    return parsed_args[0], parsed_args[1:], options.__dict__
